// Generated by CoffeeScript 1.10.0
(function() {
  var KicadGenerator, QedaElement, QedaLibrary, fs, log, mkdirp, path, request, yaml;

  fs = require('fs');

  mkdirp = require('mkdirp');

  path = require('path');

  request = require('sync-request');

  yaml = require('js-yaml');

  KicadGenerator = require('./kicad-generator');

  QedaElement = require('./qeda-element');

  log = require('./qeda-log');

  QedaLibrary = (function() {
    function QedaLibrary(config) {
      if (config == null) {
        config = {};
      }
      this.output = 'kicad';
      this.symbol = {
        style: 'default',
        gridSize: 2.5,
        fontSize: {
          "default": 2.5,
          refDes: 2.5,
          name: 2.5,
          pin: 2.5
        },
        lineWidth: {
          "default": 0,
          thick: 0.8,
          thin: 0.2
        },
        space: {
          pin: 2
        }
      };
      this.pattern = {
        style: 'default',
        densityLevel: 'N',
        decimals: 3,
        polarityMark: 'dot',
        tolerance: {
          "default": 0.1,
          fabrication: 0.1,
          placement: 0.1
        },
        clearance: {
          padToSilk: 0.2,
          padToPad: 0.2,
          padToMask: 0.05
        },
        lineWidth: {
          "default": 0.2,
          silkscreen: 0.12,
          assembly: 0.1,
          courtyard: 0.05
        },
        fontSize: {
          "default": 1,
          refDes: 1.2,
          value: 1
        },
        ball: {
          collapsible: true
        }
      };
      this.mergeObjects(this, config);
      this.elements = [];
    }

    QedaLibrary.prototype.add = function(element) {
      var aliases, base1, def, housing, housings, i, j, key, keys, len, len1, name, newElement, res, suffixes;
      def = this.load(element);
      if (def.abstract) {
        console.error("'" + element + "': Cannot add abstract component, use it only as base for others");
        process.exit(1);
      }
      res = [];
      housings = [];
      if (typeof def.housing === 'string') {
        keys = def.housing.replace(/\s+/g, '').split(',');
        for (i = 0, len = keys.length; i < len; i++) {
          key = keys[i];
          housing = def[key];
          if (housing.suffix == null) {
            housing.suffix = key;
          }
          housings.push(housing);
        }
      } else {
        if ((base1 = def.housing).suffix == null) {
          base1.suffix = '';
        }
        housings.push(def.housing);
      }
      name = def.name;
      for (j = 0, len1 = housings.length; j < len1; j++) {
        housing = housings[j];
        suffixes = housing.suffix.replace(/\s+/g, '').split(',');
        def.name = name + suffixes[0];
        def.housing = housing;
        if (suffixes.length > 1) {
          aliases = [];
          if (def.alias != null) {
            aliases.concat(def.alias.replace(/\s+/g, '').split(','));
          }
          aliases.concat(suffixes.map((function(_this) {
            return function(a) {
              return name + a;
            };
          })(this)));
          def.aliases = aliases;
        }
        newElement = new QedaElement(this, def);
        this.elements.push(newElement);
        res.push(newElement);
      }
      return res;
    };

    QedaLibrary.prototype.generate = function(name) {
      var generator;
      log.start("Render library '" + name + "'");
      this.render();
      log.ok();
      generator = null;
      switch (this.output) {
        case 'kicad':
          generator = new KicadGenerator(this);
      }
      if (generator) {
        log.start("Generate output for '" + name + "'");
        generator.generate(name);
        return log.ok();
      }
    };

    QedaLibrary.prototype.load = function(element, force) {
      var base, baseDef, baseElement, bases, def, elementYaml, error, error1, error2, i, len, localFile, res;
      if (force == null) {
        force = false;
      }
      elementYaml = element.toLowerCase() + '.yaml';
      localFile = './library/' + elementYaml;
      if ((!fs.existsSync(localFile)) || force) {
        log.start("Load '" + element + "'");
        elementYaml = elementYaml.split('/').map(function(a) {
          return encodeURIComponent(a);
        }).join('/');
        try {
          res = request('GET', 'https://raw.githubusercontent.com/qeda/library/master/' + elementYaml, {
            timeout: 3000
          });
        } catch (error1) {
          error = error1;
          log.error(error.message);
        }
        if (res.statusCode === 200) {
          mkdirp.sync(path.dirname(localFile));
          fs.writeFileSync(localFile, res.body);
          log.ok();
        } else {
          log.warning(res.statusCode);
        }
      }
      log.start("Read '" + element + "'");
      try {
        def = yaml.safeLoad(fs.readFileSync(localFile));
      } catch (error2) {
        error = error2;
        log.error("" + error.message);
      }
      if (def.base != null) {
        if (typeof def.base !== 'string') {
          def.base = def.base.toString();
        }
        bases = def.base.replace(/\s+/g, '').split(',');
        delete def.base;
        for (i = 0, len = bases.length; i < len; i++) {
          base = bases[i];
          baseElement = base;
          if (path.dirname(baseElement) === '.') {
            baseElement = path.dirname(element) + '/' + baseElement;
          }
          baseDef = this.load(baseElement);
          if (baseDef.abstract) {
            delete baseDef.abstract;
          }
          this.mergeObjects(baseDef, def);
          def = baseDef;
        }
      }
      log.ok();
      return def;
    };

    QedaLibrary.prototype.mergeObjects = function(dest, src) {
      var k, results, v;
      results = [];
      for (k in src) {
        v = src[k];
        if (typeof v === 'object' && (!Array.isArray(v)) && dest.hasOwnProperty(k)) {
          results.push(this.mergeObjects(dest[k], v));
        } else {
          results.push(dest[k] = v);
        }
      }
      return results;
    };

    QedaLibrary.prototype.render = function() {
      var element, i, len, ref;
      if (this._rendered == null) {
        this._rendered = false;
      }
      if (this._rendered) {
        return;
      }
      ref = this.elements;
      for (i = 0, len = ref.length; i < len; i++) {
        element = ref[i];
        element.render();
      }
      return this._rendered = true;
    };

    return QedaLibrary;

  })();

  module.exports = QedaLibrary;

}).call(this);
